#include "FreqSetter.h"

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#include <unistd.h>

FILE** pSetFiles = NULL;
int nbSetFiles = 1;

char openFreqSetterFiles(int cpuID)
{
   if ( cpuID != -1 )
   {
      pSetFiles = malloc(sizeof(FILE*));
      nbSetFiles = 1;
   }
   else
   {
      int nbprocs = sysconf(_SC_NPROCESSORS_ONLN);
      if ( nbprocs < 1 )
      {
         fprintf(stdout,"Fail to get the number of CPU\n");
         nbprocs = 1;
      }
      
      pSetFiles = malloc(sizeof(FILE*) * nbprocs);
      nbSetFiles = nbprocs;
   }
   
   if ( pSetFiles )
   {
   }
   else
   {
      fprintf(stdout,"Fail to allocate memory for files\n");
      return -1;
   }
   
   if ( cpuID != -1 )
   {
      char buf[200] = {'\0'};
      
      sprintf(buf,"/sys/devices/system/cpu/cpu%d/cpufreq/scaling_setspeed",cpuID);
      pSetFiles[0] = fopen(buf,"w");
      if ( pSetFiles[0] == NULL )
      {
         fprintf(stderr,"Fail to open file '%s'\n",buf);
         return -2;
      }
      
      // fprintf(stderr,"Opened : '%s'\n",buf);
   }
   else
   {
      int i = 0;
      for ( i = 0 ; i < nbSetFiles ; i++ )
      {
         char buf[200] = {'\0'};
      
         sprintf(buf,"/sys/devices/system/cpu/cpu%d/cpufreq/scaling_setspeed",i);
         pSetFiles[i] = fopen(buf,"w");
         if ( pSetFiles[i] == NULL )
         {
            fprintf(stderr,"Fail to open file '%s'\n",buf);
            return -2;
         }
         // fprintf(stderr,"Opened : '%s'\n",buf);
      }
   }
   
   return 0;
}

char setAllFreq(int targetFreq)
{
   int i = 0;
   
   for ( i = 0 ; i < nbSetFiles ; i++ )
   {
      fseek(pSetFiles[i],0,SEEK_SET);
      fprintf(pSetFiles[i],"%d",targetFreq);
      fflush(pSetFiles[i]);
   }

   return 0;
}


void closeFreqSetterFiles(void) {
   int i = 0;
   for ( i = 0 ; i < nbSetFiles ; i++ )
   {
      fclose(pSetFiles[i]);
   }
   
   free(pSetFiles);
}

void setCPUGovernor(const char* newPolicy)
{
   int nbprocs = sysconf(_SC_NPROCESSORS_ONLN);
   int i = 0;
   
   assert(newPolicy);
   
   if ( nbprocs < 1 )
   {
      fprintf(stdout,"Fail to get the number of CPU\n");
      nbprocs = 1;
   }
   
   for ( i = 0 ; i < nbprocs ; i++ )
   {
      FILE* pGovernorFile = NULL;
      
      char buf[200] = {'\0'};
      
      sprintf(buf,"/sys/devices/system/cpu/cpu%d/cpufreq/scaling_governor",i);
      pGovernorFile = fopen(buf,"w");
      if ( pGovernorFile == NULL )
      {
         fprintf(stderr,"Fail to open file '%s'\n",buf);
         continue;
      }
      
      fprintf(pGovernorFile,"%s",newPolicy);
      
      fclose(pGovernorFile);
   }
}
