#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <omp.h>

#include "FreqSetter.h"
#include "loop_memory.h"
#include "probe_msr.h"
#include "probe_time.h"

#define ITERATIONS 100000
#define N_MEM 40000

//double a[N_MEM] = {2.0}, b[N_MEM][20] = {{0.0}};

void init_vector(unsigned int n, double* vector, double value)
{
	unsigned int i;
	for (i = 0; i < n; i ++)
	{
		vector[i] = value;
	}
}

int main (int argc, char* argv[])
{
	unsigned int freq = 0;
	unsigned int iterations=ITERATIONS;
	unsigned int vectorSize=N_MEM;
	double *a, *b;
   double startPower=0;
   double endPower=0;
   long long unsigned startTime=0;
   long long unsigned endTime=0;

	if (argc == 2)
	{
		sscanf(argv[1], "%u", &freq);
	}
   else
   {
      fprintf(stdout,"Veuillez donner la fréquence à laquelle faire le test\n");
      return 0;
   }
   
   // Allocation de mémoire pour le test
	a = malloc(vectorSize*32 * sizeof(double));
	b = malloc(vectorSize*32 * sizeof(double));
	init_vector(vectorSize*32, a, 0.0);
	init_vector(vectorSize*32, b, 2.0);
	
   setCPUGovernor("userspace");

   // Ouvre les fichiers de fréquence
   if ( openFreqSetterFiles(-1) != 0 )
   {
      return -3;
   }

   // Définit la fréquence voulue
   setAllFreq(freq);
   
   // Attends quelques temps pour être certain de l'efficacité du changement
   usleep(100);
   
   int msrFD = open_msr(0);
   if ( msrFD > 0 )
   {
      // Première mesure
      startTime = getTime();
      startPower = readPower(msrFD);
      
      // Exécution du test
      test(vectorSize, a, b, iterations);
      
      // Seconde mesure
      endPower = readPower(msrFD);
      endTime = getTime();
      
      // On affiche la différence entre les deux mesures pour avoir une consommation
      fprintf(stdout,"Energie : %f (J) | Temps : %llu (ms) | Puissance : %f (W)\n",endPower-startPower,endTime-startTime,(endPower-startPower)/((double)(endTime-startTime)/1000.0));
      
      // Nettoyage
      close(msrFD);
   }

   setCPUGovernor("ondemand");
   closeFreqSetterFiles();

	free(a);
	free(b);

	return 0; 
}
